/************************************************************************
 *
 * $Id: poller.h,v 1.13 2002/03/19 00:36:11 crooney Exp $
 *
 ***********************************************************************/

/**************************************************************************
Copyright 2001, Christopher Rooney crooney@crooney.com

This file is part of fpoller.

fpoller is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

fpoller is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with fpoller; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/

#ifndef POLLER_H
#define POLLER_H

#include "poller_util.h"

///entire library is contained in this namespace
namespace fpoller {
  extern const short WRITE; 
  extern const short READ;

  /**  This is the main class in the library.  It delegates all logic to its delegate_ member.
       It is generally referred to by supplied typedef -- poller, fast_poller, or traditional_poller.
   */
  template<typename Delegate> 
    class basic_poller {
    private:
      Delegate delegate_;
    public:
      /** Default constructor */
      basic_poller():delegate_(){}
      /** Registers a callback for a file descriptor for a particular interest (READ or WRITE) */
      void add(int fd,short interest, callback * cb){ delegate_.add(fd,interest,cb); }
      /** Removes a callback for a file descriptor for a particular interest (READ or WRITE) */
      void remove(int fd,short interest){ delegate_.remove(fd,interest); }
      /** Removes a callback for a file descriptor for both READ and WRITE*/
      void remove(int fd){ remove(fd,WRITE); remove(fd,READ); }
      /** Checks for ready file descriptors and executes the callbacks associated with them.
       Will wait for timeout milliseconds or until some descriptora are ready*/
      int poll(int timeout){return delegate_.poll(timeout);}
    };
}

#include "poller_generic.h"
namespace fpoller { 
  /** Always available */
  typedef basic_poller<poller_generic> traditional_poller;
}

#ifdef POLLER_DEVPOLL
#include "poller_devpoll.h"
namespace fpoller { 
  typedef basic_poller<poller_devpoll> fast_poller;
  typedef fast_poller poller;
}
#endif

#ifdef POLLER_KQUEUE
#include "poller_kqueue.h"
namespace fpoller { 
  typedef basic_poller<poller_kqueue> fast_poller;
  typedef fast_poller poller;
}
#endif

#ifndef POLLER_KQUEUE
#ifndef POLLER_DEVPOLL
namespace fpoller{ 
  /** poller is a synonym for basic_poller<delegate> where 
      delegate is one of poller_devpoll or poller_kqueue 
      on platforms where they are available.  If neither 
      is available it is typedefed as seen here. */
  typedef traditional_poller poller;
}
#endif
#endif

#endif //POLLER_H






