/************************************************************************
 *
 * $Id: poller_devpoll.cc,v 1.14 2001/08/21 19:57:52 crooney Exp $
 *
 ***********************************************************************/

/**************************************************************************
Copyright 2001, Christopher Rooney crooney@crooney.com

This file is part of fpoller.

fpoller is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

fpoller is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with fpoller; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/

#include <string.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/poll.h>
#include <sys/devpoll.h>
#include <errno.h>

#include "poller_devpoll.h"
#include "poller_util.h"

namespace fpoller {

  poller_devpoll::poller_devpoll():pollfds_(MAX_FDS){
    if ((devpoll_fd_ = open("/dev/poll",O_RDWR)) <0) {
      throw new exception("Could not open /dev/poll",errno);
    }
    dvpoll_.dp_fds = &pollfds_[0];
    dvpoll_.dp_nfds= MAX_FDS;
    dvpoll_.dp_timeout = 0;
  }

  poller_devpoll::~poller_devpoll(){
    close(devpoll_fd_);
  }

  void poller_devpoll::add(int fd, short interest, callback *cb){
    if (cb->operator()(fd) == REMOVE) {
      //we can just avoid adding;
      delete cb;
      return;
    }
    pollfd pfd;
    pfd.fd = fd;
    pfd.events=interest;
    changefds_.push_back(pfd);      
    fd_maps_.get_map(interest).insert(make_pair(fd,callback_wrapper(cb)));
  }
  
  void poller_devpoll::remove(int fd, short interest){
    changefds_.push_back((struct pollfd){fd,POLLREMOVE,0});
    fd_maps_.get_map(interest).erase(fd);
    // if we are removing for READ, but we still want WRITE, we have to resubmit and vice versa
    short opposite_interest = interest & WRITE ? READ : WRITE;
    fd_map &map = fd_maps_.get_map(opposite_interest);
    if ( map.find(fd) != map.end()){
      pollfd pfd;
      pfd.fd = fd;
      pfd.events=opposite_interest;
      changefds_.push_back(pfd);      
    }
  }

  int poller_devpoll::poll(int timeout){
    //first take care of all the changes that have been made since the last time we polled
    if (changefds_.size() > 0){
      if (write(devpoll_fd_,&changefds_[0],(sizeof(pollfd) * changefds_.size())) != static_cast<int>(sizeof(pollfd) * changefds_.size())){
	throw exception("Could not write to /dev/poll",errno);
      }
    }
    changefds_.clear();
    int nready,retval;
    dvpoll_.dp_timeout = timeout;
    if ((retval = nready = ioctl(devpoll_fd_,DP_POLL,&dvpoll_)) < 0){
      throw exception("Could not read from /dev/poll",errno);
    }
    for (pollfd * pfd = &pollfds_[0]; nready ; pfd++){
      nready--;
      fd_map &map = fd_maps_.get_map(pfd->revents);
      fd_map::iterator it = map.find(pfd->fd);
      if (it == map.end()){
	//should never happen
	remove(pfd->fd,pfd->revents);
      }else{
	if ((*(it->second))(pfd->fd) == REMOVE){
	  remove(pfd->fd,pfd->revents);
	}
      }
    }
    return retval;
  }
  
}//namespace fpoller

