/************************************************************************
 *
 * $Id: poller_devpoll.h,v 1.11 2001/08/19 00:31:53 crooney Exp $
 *
 ***********************************************************************/

/**************************************************************************
Copyright 2001, Christopher Rooney crooney@crooney.com

This file is part of fpoller.

fpoller is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

fpoller is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with fpoller; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/

#ifndef POLLER_DEVPOLL_H
#define POLLER_DEVPOLL_H

#include <unistd.h>
#include <poll.h>
#include <hash_map>
#include <sys/devpoll.h>

#include "poller_util.h"

namespace fpoller {
  /** Delegate poller class for systems supporting /dev/poll mechanism
      (Solaris 8 and patched Linux kernels).  Supports same public operations 
      as basic_poller().  Should never be instantiated by client code.
      The typedef poller will refer to basic_poller&lt;poller_kqueue&gt;
      on systems on which this class is available.
   */
  class poller_devpoll {
  private:
    vector<struct pollfd> pollfds_, changefds_;
    typedef std::hash_map<int, callback_wrapper> fd_map;
    struct fd_maps {
      fd_map reader,writer;
      inline fd_map &get_map(short interest){return (interest & WRITE) ? writer : reader;}
    };
    fd_maps fd_maps_;
    //int num_changed_;
    int devpoll_fd_;
    dvpoll dvpoll_;
  public:
    /** Opens the /dev/poll driver.  Throws fpoller::exception if the open fails.*/
    poller_devpoll();
    /** Closes the /dev/poll driver.*/
    ~poller_devpoll();
    void add(int fd, short events, callback *cb);///<Implementation of basic_poller::add.
    void remove(int fd,short interest);///<Implementation of basic_poller::remove.
    int poll(int timeout);///<Implementation of basic_poller::poll.
  };
}//namespace

#endif
