/************************************************************************
 *
 * $Id: poller_generic.cc,v 1.13 2002/10/16 19:36:38 crooney Exp $
 *
 ***********************************************************************/

/**************************************************************************
Copyright 2001, Christopher Rooney crooney@crooney.com

This file is part of fpoller.

fpoller is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

fpoller is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with fpoller; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/

#include "poller_generic.h"

namespace fpoller {
  poller_generic::poller_generic():num_fds_(0),pollfds_(MAX_FDS){
    for (int i = 0; i<MAX_FDS; i++){
      available_.push(i);
      pollfds_[i].fd = -1;
    }
  }

  void poller_generic::add(int fd, short interest, callback* cb){
    if (cb->operator()(fd) == REMOVE) {
      //we can just avoid adding;
      delete cb;
      return;
    }
    if (available_.empty()){
      unsigned i = pollfds_.size();
      pollfds_.resize(static_cast<unsigned>(i*1.5));
      for (;i < pollfds_.size();i++){
	available_.push(i);
	pollfds_[i].fd = -1;
      }
    }else{
      fd_map::iterator iter = fd_map_.find(fd);
      if (iter != fd_map_.end()){
	fd_info &info = iter->second;
	pollfds_[info.pollfds_index].fd = fd;
	pollfds_[info.pollfds_index].events |= interest;
	if ((interest & READ)){
	  info.reader = callback_wrapper(cb);
	}
	if ((interest & WRITE)){
	  info.writer = callback_wrapper(cb);
	}
      }else{
	int index = available_.top();
	available_.pop();
	pollfds_[index].fd = fd;
	pollfds_[index].events = interest;
	fd_info info;
	info.pollfds_index = index;
	if (interest & READ){
	  info.reader = callback_wrapper(cb);
	}
	if (interest & WRITE){
	  info.writer = callback_wrapper(cb);
	}
	fd_map_.insert(std::make_pair(fd,info));
	if (index > num_fds_){
	  num_fds_ = index;
	}
      }
    }
  }

  void poller_generic::remove(int fd, short interest){
    fd_map::iterator iter = fd_map_.find(fd);
    if (iter != fd_map_.end()){
      fd_info &info = iter->second;
      int &index = info.pollfds_index;
      pollfd &pfd = pollfds_[index];
      if (interest ^ pfd.events > 0){
	//the fd waits on other events just remove for interest
	pfd.events ^= interest;
	if (interest & READ){
	  info.reader = callback_wrapper();
	}
	if (interest & WRITE){
	  info.writer = callback_wrapper();
	}
      }else{
	//actually remove
	pfd.fd = -1;
	pfd.events = 0;
	pfd.revents = 0;
	available_.push(index);
	if (index >= num_fds_){
	  for(;pollfds_[num_fds_].fd <0 && num_fds_;){
	    num_fds_--;
	  }
	}
	fd_map_.erase(iter);
      }
    }
  }

  int poller_generic::poll(int timeout){
    int nready,retval;
    if ((retval = nready = ::poll(&pollfds_[0],num_fds_+1,timeout)) < 0){
      return retval;
    }

    for (pollfd *pfd = &pollfds_[0];
	 nready && (pfd < (&pollfds_[num_fds_ + 1])) ;
	 pfd++){
      short relevant_events;
      if ((relevant_events = pfd->revents & (pfd->events | ERROR))){
	nready--;
	fd_info &info = fd_map_[pfd->fd];
	if (relevant_events & (READ | ERROR)) {
	  try_callback(pfd->fd,info.reader,READ);
	}
	if (relevant_events & (WRITE | ERROR)) {
	  try_callback(pfd->fd,info.writer,WRITE);
	}
      }
    }
    return retval;
  }
  
}


  
