/************************************************************************
 *
 * $Id: poller_generic.h,v 1.12 2002/10/16 19:36:38 crooney Exp $
 *
 ***********************************************************************/

/**************************************************************************
Copyright 2001, Christopher Rooney crooney@crooney.com

This file is part of fpoller.

fpoller is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

fpoller is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with fpoller; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/

#ifndef POLLER_GENERIC_H
#define POLLER_GENERIC_H

#include <unistd.h>
#include <poll.h>
#include <vector>
#include <hash_map>
#include <queue>

#include "poller_util.h"

namespace fpoller {
  /** Default delegate poller class.  Should work on all UNIX systems.
      basic_poller&lt;poller_generic&gt; will always be typedefed as traditional_poller.
      If no faster polling mechanism exists on the system poller will also by typedefed
      to basic_poller&lt;poller_generic&gt; */
  class poller_generic {
  private:
    struct fd_info {
      callback_wrapper reader,writer;
      int pollfds_index;
    };
    typedef std::hash_map<int,fd_info> fd_map;
    fd_map fd_map_;
    int num_fds_;
    std::vector<pollfd> pollfds_;
    std::priority_queue<int,std::vector<int>,std::greater<int> > available_;
    inline void try_callback(int fd, callback_wrapper &cb, int interest){if (cb.operator->() && cb(fd) == REMOVE) remove(fd,interest);}    
  public:
    poller_generic();
    void add(int fd, short interest, callback* cb);///<Implementation of basic_poller::add.
    void remove(int fd, short interest);///<Implementation of basic_poller::remove.
    int poll(int timeout);///<Implementation of basic_poller::poll.
  };

}

#endif //GENERIC_POLLER_H
