/************************************************************************
 *
 * $Id: poller_kqueue.cc,v 1.13 2002/10/16 19:36:38 crooney Exp $
 *
 ***********************************************************************/

/**************************************************************************
Copyright 2001, Christopher Rooney crooney@crooney.com

This file is part of fpoller.

fpoller is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

fpoller is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with fpoller; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/

#include <string.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/event.h>
#include <fcntl.h>
#include <sys/poll.h>
#include <errno.h>

#include "poller_kqueue.h"

namespace fpoller {
  
  void poller_kqueue::add(int fd, short interest, callback *cb){
    if (cb->operator()(fd) == REMOVE) {
      //we can just avoid adding;
      delete cb;
      return;
    }
    struct kevent k;
    EV_SET(&k,fd,poll_to_kq(interest),EV_ADD,0,0,0);
    change_queue_.push_back(k);
    fd_maps_.get_map(interest).insert(std::make_pair(fd,callback_wrapper(cb)));
  }

  void poller_kqueue::remove(int fd, short interest){
    fd_map &map = fd_maps_.get_map(interest);
    fd_map::iterator it = map.find(fd);
    if (it != map.end()){
      struct kevent k;
      EV_SET(&k,fd,poll_to_kq(interest),EV_DELETE,0,0,0);
      map.erase(it);
      change_queue_.push_back(k);
    }
  }

  int poller_kqueue::poll(int timeout){
    timespec ts;
    ts.tv_sec = timeout / 1000;
    ts.tv_nsec = (timeout % 1000) * 1000;
    int nready,retval;
    if (poll_queue_.size() < (fd_maps_.size()+(change_queue_.size()))){
      // after call to remove, we need room for an error report for 
      // the EV_DELETE, but fd is not in fd_map_ 
      poll_queue_.resize(fd_maps_.size()+(change_queue_.size()));
    }
    if ((retval = nready = ::kevent(kq_fd_,&change_queue_[0],change_queue_.size(),
				  &poll_queue_[0],poll_queue_.size(),&ts)) > -1){
      change_queue_.clear();
      for (struct kevent *ke = &poll_queue_[0]; nready ; ke++){
	nready--;
	short interest = kq_to_poll(ke->filter);	
	fd_map &map = fd_maps_.get_map(interest);	
	fd_map::iterator it = map.find(ke->ident);
	if (it == map.end()){
	  //should never happen
	  remove(ke->ident,interest);
	}else{
	  if ((*(it->second))(ke->ident) == REMOVE){
	    remove(ke->ident,interest);
	  }
	}
      }
    }
    return retval;
  }
}//namespace






