/************************************************************************
 *
 * $Id: poller_kqueue.h,v 1.13 2002/10/16 19:36:38 crooney Exp $
 *
 ***********************************************************************/

/**************************************************************************
Copyright 2001, Christopher Rooney crooney@crooney.com

This file is part of fpoller.

fpoller is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

fpoller is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with fpoller; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/
#ifndef POLLER_KQUEUE_H
#define POLLER_KQUEUE_H

#include <sys/types.h>
#include <sys/event.h>
#include <vector>
#include <hash_map>

#include "poller_util.h"

namespace fpoller {
  /** Delegate poller class for systems supporting kqueue() and kevent() 
      (currently FreeBSD and OpenBSD).  Supports same public operations 
      as basic_poller().  Should never be instantiated by client code.
      The typedef poller will refer to basic_poller&lt;poller_kqueue&gt;
      on systems on which this class is available.
   */
  class poller_kqueue {
  private:
    int kq_fd_;
    std::vector<struct kevent> poll_queue_,change_queue_;
    typedef std::hash_map<int, callback_wrapper> fd_map;
    struct fd_maps {
      fd_map reader,writer; 
      inline int size(){return reader.size()+writer.size();}
      inline fd_map &get_map(short interest){return (interest & WRITE) ? writer : reader;}
    };
    fd_maps fd_maps_;
    inline static short poll_to_kq(short filter){return filter==WRITE ? EVFILT_WRITE : EVFILT_READ;}
    inline static short kq_to_poll(short filter){return filter==EVFILT_WRITE ? WRITE : READ;}
  public:
    /** Acquires descriptor from kqueue().  Throws fpoller::exception if it can't. */
    poller_kqueue(){if ((kq_fd_ = kqueue()) <0 ) throw exception("could not open kqueue",errno);}
    /** Releases kqueue descriptor. */
    ~poller_kqueue(){close(kq_fd_);}
    void add(int fd, short events, callback *cb);///<Implementation of basic_poller::add.
    void remove(int fd,short interest);///<Implementation of basic_poller::remove.
    int poll(int timeout);///<Implementation of basic_poller::poll.
  };
}

#endif
