/************************************************************************
 *
 * $Id: poller_util.cc,v 1.11 2001/08/19 02:35:44 crooney Exp $
 *
 ***********************************************************************/

/**************************************************************************
Copyright 2001, Christopher Rooney crooney@crooney.com

This file is part of fpoller.

fpoller is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

fpoller is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with fpoller; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*************************************************************************/

#include <string.h>
#include <unistd.h>
#include <cstdio>
#include <errno.h>

#include "poller_util.h"
#include "poller.h"

namespace fpoller{

  int callback_wrapper::operator()(int fd){
    if (cb_.ptr()) {
      return cb_->operator()(fd); 
    } else {
      return REMOVE;
    }
  }

  int get_listen_socket(int port, int backlog){
    int sock; 
    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0){
      throw exception("could not create listen socket",errno);
    }
    sockaddr_in sa;
    //this was fun to find.  breaks without this line on FreeBSD
    memset(&sa,0,sizeof(sockaddr_in));

    sa.sin_family = AF_INET;
    /* are you ready for some bull%@$%?
       gcc 2.95.2 chokes on "asm volatile" inside templates
       on linux intel and this is how htons is defined when optimization is turned on. 
       so use c endian swap
    */
#if defined __OPTIMIZE__ && defined __i386__ && defined __linux__
    sa.sin_port = ((((port) >> 8) & 0xff) | (((port) & 0xff) << 8));
#else
    sa.sin_port = htons(port);
#endif
    sa.sin_addr.s_addr = htonl(INADDR_ANY);
	
    //set the address to be reusable so we can shut down and restart with getting bind errors
    int one=1;
    if (setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,&one,sizeof(int)) != 0){
      throw exception("could not set socket to be reusable",errno);
    }
    if (bind(sock,(struct sockaddr *)&sa, sizeof(sa)) < 0) {
      throw exception("could not bind listen socket",errno);
    }
    if (listen(sock,backlog) < 0) {
      throw exception("could not start listen socket listening",errno);
    }
    return sock;
  }

  void set_non_blocking(int fd){
    int flags = fcntl(fd,F_GETFL,0);
    if (flags<0) {
      throw exception("could not get socket's flags to set it non-blocking",errno);
    }
    if (fcntl(fd,F_SETFL,flags | O_NONBLOCK) <0){
      throw exception("could not set socket non-blocking",errno);
    }
  }

}//namespace fpoller



