#ifndef UTIL_TYPES_H
#define UTIL_TYPES_H

#include <vector>
#include <iostream>
#include <hash_set> //to get std::hash
#include <numeric>  //to_get accumulate
#include "SmartPtr.h"

namespace fpoller {

  namespace util_types {
    /** Used most places in preference to const char buffers */
    typedef std::vector<char> vec;
    /** Just a smart pointer to vec */
    typedef Loki::SmartPtr<vec> svec;

    /** Helper class for svec_hash */
    template<typename T, typename U>
    struct hashing_accumulator{
      //pretty much what the sgi hash<const char *> does
      T operator()(T t,U u){
	return 5*t+u;
      }
    };
    
    /**Hashing class for svec for use in std::hash_map<svec, T> or std::hash_set<svec>.*/
    struct svec_hash {
      size_t operator()(const svec &s) const{
	return std::accumulate(s->begin(),s->end(),0,hashing_accumulator<size_t,char>());
      }
    };
    
    /**Syntactic sugar for converting svec to const char *. Must be used explicitly.  Automatic conversion unwanted.*/
    inline const char *pcc(const svec &s){
      return &((*s)[0]);
    }

    /** construct a vec pointer (which may then intitalize an svec) from a null terminated string. */
    inline vec * new_vec_from_cstring(const char * pc){
      return new vec(pc,pc+strlen(pc));
    }


    inline void to_cerr(const svec & s, char c = ' '){
#ifdef DEBUG
      std::ostream_iterator<char> out(std::cerr);
      copy(s->begin(),s->end(),out);
      *out++ = c;
#endif
    }

    /** Character by character comparison of svec and const char */
    inline bool equals(const svec & s, const char * c){
      size_t matched = 0;
      for(vec::iterator i = s->begin();(*c) && i != s->end() && *i++ == *c++;matched++);
      return matched == s->size();
    }

    /** Character by character comparison of svec and const char */
    inline bool equals(const char * c,const svec & s){
      return equals(s,c);
    }
    

  }// namespace util_types

}//namespace fpoller

namespace std {
  /** Allow vecs to be written to streams. */
  inline std::ostream & operator<<(std::ostream &o, const fpoller::util_types::vec &v){
    return o.write(&v[0],v.size());
  }
}
#endif
