// BalanceSheet.cpp
// The Dtl Sample for the article on adaptable size classes
// by Julian Bushell
//
// Note to use this sample, download Dtl from the web and link with its library.
// Then set up an ODBC connection for the supplied balance_sheets.mdb

#include "stdafx.h"

#include <iostream>
#include <vector>
using namespace std;

#include "dtl.h"
using namespace dtl;


template <class I>
struct DtlConfig {
	template<class I> struct BCAObj {
		void operator()(BoundIOs &cols, I &rowbuf)
		{
			rowbuf.bindColumns(cols);
		}
	};
};


// when used with an attribute class definition, it helps define a unique class type
struct Unique_ {};

// used as the deepest nested class in the definition of class variations
struct Base_ {
	
	template <class T> Base_(const T &) {}
	Base_() {}
	
	void set(...) const {}
	
	void bindColumns(BoundIOs &cols) {}
};


// attribute class Stock
template <class Parent> struct Stock_: public Parent {
	
	template <class T> Stock_(const T & t_): Parent(t_) { t_.set(this->stock, Stock_<Unique_>()); }
	
	template <class T> Stock_ & operator=(const T & t_)
	{
		Parent::operator=(t_);
		t_.set(this->stock, Stock_<Unique_>());
		
		return *this;
	}
	
	template <class T> void set(double & t_, T t) const { Parent::set(t_, t); }		
	void set(double & stock, Stock_<Unique_>) const { stock = this->stock; }
	
	void bindColumns(BoundIOs &cols)
	{
		cols["stock"] == this->stock;
		Parent::bindColumns(cols);
	}
	
	Stock_() {}
	
	double stock;
};


// attribute class Debtors
template <class Parent> struct Debtors_: public Parent {
	
	template <class T> Debtors_(const T & t_): Parent(t_) { t_.set(this->debtors, Debtors_<Unique_>()); }
	
	template <class T> Debtors_ & operator=(const T & t_)
	{
		Parent::operator=(t_);
		t_.set(this->debtors, Debtors_<Unique_>());
		
		return *this;
	}
	
	template <class T> void set(double & t_, T t) const { Parent::set(t_, t); }		
	void set(double & debtors, Debtors_<Unique_>) const { debtors = this->debtors; }
	
	void bindColumns(BoundIOs &cols)
	{
		cols["debtors"] == this->debtors;
		Parent::bindColumns(cols);
	}
	
	Debtors_() {}
	
	double debtors;
};


// attribute class Cash
template <class Parent> struct Cash_: public Parent {
	
	template <class T> Cash_(const T & t_): Parent(t_) { t_.set(this->cash, Cash_<Unique_>()); }
	
	template <class T> Cash_ & operator=(const T & t_)
	{
		Parent::operator=(t_);
		t_.set(this->cash, Cash_<Unique_>());
		
		return *this;
	}
	
	template <class T> void set(double & t_, T t) const { Parent::set(t_, t); }		
	void set(double & cash, Cash_<Unique_>) const { cash = this->cash; }
	
	void bindColumns(BoundIOs &cols)
	{
		cols["cash"] == this->cash;
		Parent::bindColumns(cols);
	}
	
	Cash_() {}
	
	double cash;
};




int main(int argc, char* argv[])
{
	DBConnection::GetDefaultConnection().Connect("DSN=balance_sheets");
	
	// define class variations
	typedef  Cash_< Debtors_<Stock_<Base_> > > CurrentAssets;
	typedef Debtors_<Base_> Debtors;
	
	vector<CurrentAssets> currentAssetsVec;
	vector<Debtors> debtorsVec;

	// open balance_sheets table
	DBView<CurrentAssets> view("balance_sheets", DtlConfig<CurrentAssets>::BCAObj<CurrentAssets>());
	
	// read in CurrentAssets objects from database table
	for (DBView<CurrentAssets>::select_iterator readIt = view.begin(); readIt != view.end();  ++readIt)
		currentAssetsVec.push_back(*readIt);

	// show the CurrentAssets objects Debtors attributes
	cout << "From CurrentAssets objects..." << endl;
	for (vector<CurrentAssets>::const_iterator currentAssetsIt=currentAssetsVec.begin(); currentAssetsIt != currentAssetsVec.end();  ++currentAssetsIt)
		cout << "debtors attribute = " << currentAssetsIt->debtors << endl;
	
	// copy CurrentAssets objects to Debtors objects
	copy (currentAssetsVec.begin(), currentAssetsVec.end(), back_inserter(debtorsVec));

	// now clear this lot so there is only debtorsVec
	currentAssetsVec.clear();

	// can load more debtorsVec objects with available memory since they are smaller than currentAssetsVec objects
	// .
	// .
	
	// show that Debtors objects do indeed have the Debtors values from CurrentAssets objects Debtors attributes
	cout << endl << "From Debtors objects..." << endl;
	for (vector<Debtors>::const_iterator debtorsIt=debtorsVec.begin(); debtorsIt != debtorsVec.end();  ++debtorsIt)
		cout << "debtors attribute = " << debtorsIt->debtors << endl;
	
	return 0;
}
