// CBA.cpp
// Sample for Listing1 in the article on adaptable size classes
// by Julian Bushell

#include "stdafx.h"

#include <iostream>
using namespace std;


// when used with an attribute class definition, it helps define a unique class type
struct Unique_ {};

// used as the deepest nested class in the definition of class variations
struct Base_ {
	
	template <class T> Base_(const T &) {}
	Base_() {}
	
	void set(...) const {}
};


// attribute class a
template <class Parent> struct A_: public Parent {
	
	template <class T> A_(const T & t_): Parent(t_) { t_.set(this->a, A_<Unique_>()); }
	
	template <class T> A_ & operator=(const T & t_)
	{
		Parent::operator=(t_);
		t_.set(this->a, A_<Unique_>());
		
		return *this;
	}
	
	template <class T> void set(double & t_, T t) const { Parent::set(t_, t); }		
	void set(double & a, A_<Unique_>) const { a = this->a; }
	
	A_() {}
	
	double a;
};


// attribute class b
template <class Parent> struct B_: public Parent {
	
	template <class T> B_(const T & t_): Parent(t_) { t_.set(this->b, B_<Unique_>()); }
	
	template <class T> B_ & operator=(const T & t_)
	{
		Parent::operator=(t_);
		t_.set(this->b, B_<Unique_>());
		
		return *this;
	}
	
	template <class T> void set(double & t_, T t) const { Parent::set(t_, t); }		
	void set(double & b, B_<Unique_>) const { b = this->b; }
	
	B_() {}
	
	double b;
};

// attribute class c
template <class Parent> struct C_: public Parent {
	
	template <class T> C_(const T & t_): Parent(t_) { t_.set(this->c, C_<Unique_>()); }
	
	template <class T> C_ & operator=(const T & t_)
	{
		Parent::operator=(t_);
		t_.set(this->c, C_<Unique_>());
		
		return *this;
	}
	
	template <class T> void set(double & t_, T t) const { Parent::set(t_, t); }		
	void set(double & c, C_<Unique_>) const { c = this->c; }
	
	C_() {}
	
	double c;
};


int main(int argc, char* argv[])
{
	// some definitions for class variations
	typedef C_<B_<A_<Base_> > > CBA;
	typedef C_<B_<Base_> > CB;
	typedef C_<A_<Base_> > CA;
	typedef A_<C_<Base_> > AC;
	typedef B_<Base_> B;
	
	// declaring objects of class variations
	CBA cba;
	CB cb;
	CA ca;
	AC ac;
	B b;

	b.b = 2;

	ca.a = 1;
	ca.c = 3;

	// assigns the values of common attributes between class variations

	cba = ca;		  // cba.a=ca.a and cba.c=ca.c
	cba = b;		  // cba.b=b.b
	cout << "cba.a, cba.b & cba.c = " << cba.a << "," << cba.b << " & " << cba.c << " respectively" << endl;

	cb = cba;         // cb.b=cba.b and cb.c=cba.c
	ac = ca;          // ac.c=ca.c and ac.a=ca.a
	cba = ac;         // cba.a=ac.a and cba.c=ac.c
	cba = cb;         // cba.b=cb.b and cba.c=cb.c

	cout << "cb.b, cb.c = " << cb.b << "," << cb.c << " respectively" << endl;
	cout << "ac.a, ac.c = " << ac.a << "," << ac.c << " respectively" << endl;
	
	return 0;
}
