/* ////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_malloc_allocator.h (originally MRAtors.h; ::SynesisCrt)
 *
 * Purpose:     stlsoft_malloc_allocator class - uses malloc()/free().
 *
 * Created:     2nd January 2001
 * Updated:     26th September 2002
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002, Synesis Software Pty Ltd.
 *
 *              All rights reserved. 
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such 
 *              damage. 
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_STLSOFT_MALLOC_ALLOCATOR
#define _STLSOFT_INCL_H_STLSOFT_MALLOC_ALLOCATOR

#define _STLSOFT_VER_H_STLSOFT_MALLOC_ALLOCATOR_MAJOR       1
#define _STLSOFT_VER_H_STLSOFT_MALLOC_ALLOCATOR_MINOR       3
#define _STLSOFT_VER_H_STLSOFT_MALLOC_ALLOCATOR_REVISION    2
#define _STLSOFT_VER_H_STLSOFT_MALLOC_ALLOCATOR_EDIT        21

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _STLSOFT_INCL_H_STLSOFT
 #include "stlsoft.h"   // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT */
#if defined(_STLSOFT_CF_THROW_BAD_ALLOC) || \
    (   defined(_MSC_VER) && \
        _MSC_VER < 1100)
#include <new>          // placement new, std::bad_alloc
#endif /* _STLSOFT_CF_THROW_BAD_ALLOC || _MSC_VER < 1100) */

/* ////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACES
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACES */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

template <ss_typename_param_k T>
class malloc_allocator;

// Specialisation for void
#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
class malloc_allocator<void>
{
public:
    typedef void                                value_type;
    typedef malloc_allocator<void>              class_type;
    typedef void                                *pointer;
    typedef void const                          *const_pointer;
    typedef ptrdiff_t                           difference_type;
    typedef ss_size_t                           size_type;

#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_CLASS_SUPPORT
    template <ss_typename_param_k U> 
    struct rebind
    {
        typedef malloc_allocator<U>             other;
    };
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_CLASS_SUPPORT */
};

template <ss_typename_param_k T>
class malloc_allocator
{
public:
    typedef T                                   value_type;
    typedef malloc_allocator<value_type>        class_type;
    typedef value_type                          *pointer;
    typedef value_type const                    *const_pointer;
    typedef value_type                          &reference;
    typedef value_type const                    &const_reference;
    typedef ptrdiff_t                           difference_type;
    typedef ss_size_t                           size_type;

#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_CLASS_SUPPORT
    template <ss_typename_param_k U> 
    struct rebind
    {
        typedef malloc_allocator<U>             other;
    };
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_CLASS_SUPPORT */

// Construction
public:
    malloc_allocator() stlsoft_throw_0()
    {}
#ifndef __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT
    malloc_allocator(const malloc_allocator &rhs) stlsoft_throw_0()
    {}
#else
    template <ss_typename_param_k U> 
    malloc_allocator(const malloc_allocator<U> &rhs) stlsoft_throw_0()
    {}
#endif /* !__STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT */
    ~malloc_allocator() stlsoft_throw_0()
    {}

// Attributes
public:
    size_type max_size() const stlsoft_throw_0()
    {
        return static_cast<size_type>(-1) / sizeof(value_type);
    }

// Conversion
public:
    pointer address(reference x) const stlsoft_throw_0()
    {
        return &x;
    }
    const_pointer address(const_reference x) const stlsoft_throw_0()
    {
        return &x; 
    }

// Allocation
public:
    pointer allocate(size_type n, malloc_allocator<void>::const_pointer = NULL)
    {
        malloc_allocator<void>::pointer p   =   malloc(n * sizeof(value_type));

#ifdef _STLSOFT_CF_THROW_BAD_ALLOC
        if(p == NULL)
        {
            throw stlsoft_ns_qual_std(bad_alloc)();
        }
#endif /* _STLSOFT_CF_THROW_BAD_ALLOC */

        return static_cast<pointer>(p);
    }

    pointer reallocate(pointer p, size_type n, malloc_allocator<void>::const_pointer = NULL)
    {
        malloc_allocator<void>::pointer p   =   realloc(p, n * sizeof(value_type));

#ifdef _STLSOFT_CF_THROW_BAD_ALLOC
        if(p == NULL)
        {
            throw stlsoft_ns_qual_std(bad_alloc)();
        }
#endif /* _STLSOFT_CF_THROW_BAD_ALLOC */

        return static_cast<pointer>(p);
    }

    void deallocate(pointer p, size_type)
    {
        free(p);
    }

    void deallocate(pointer p)
    {
        free(p);
    }

// Operations
public:
    void construct(pointer p, value_type const &x)
    {
        stlsoft_assert(p != NULL);

        new(p) value_type(x); 
    }

    void construct(pointer p)
    {
        stlsoft_assert(p != NULL);

        new(p) value_type(); 
    }

    void destroy(pointer p) stlsoft_throw_0()
    {
        stlsoft_assert(p != NULL);

        stlsoft_destroy_instance(T, value_type, p);
    }

// Not to be implemented
private:
    class_type const &operator =(class_type const &rhs);
};

template <ss_typename_param_k T>
inline ss_bool_t operator ==(const malloc_allocator<T> &/* lhs */, const malloc_allocator<T> &/* rhs */)
{
    return ss_true_v;
}

template <ss_typename_param_k T>
inline ss_bool_t operator !=(const malloc_allocator<T> &/* lhs */, const malloc_allocator<T> &/* rhs */)
{
    return ss_false_v;
}

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACES
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACES */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_STLSOFT_INCL_H_STLSOFT_MALLOC_ALLOCATOR */

/* ////////////////////////////////////////////////////////////////////////// */
