/* /////////////////////////////////////////////////////////////////////////////
 * File:        unixstl_performance_counter.h
 *
 * Purpose:     performance_counter class.
 *
 * Created:     16th January 2002
 * Updated:     2nd October 2002
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Public License)
 *
 *              Copyright (C) 2002, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/unixstl
 *                          http://www.unixstl.org/
 *
 *              email:      submissions@unixstl.org  for submissions
 *                          admin@unixstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _INCL_UNIXSTL_H_UNIXSTL_PERFORMANCE_COUNTER
#define _INCL_UNIXSTL_H_UNIXSTL_PERFORMANCE_COUNTER

#define _UNIXSTL_VER_H_UNIXSTL_PERFORMANCE_COUNTER_MAJOR        0
#define _UNIXSTL_VER_H_UNIXSTL_PERFORMANCE_COUNTER_MINOR        9
#define _UNIXSTL_VER_H_UNIXSTL_PERFORMANCE_COUNTER_REVISION     11
#define _UNIXSTL_VER_H_UNIXSTL_PERFORMANCE_COUNTER_EDIT         12

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <sys/time.h>

#if (   defined(NDEBUG) || \
        !defined(_DEBUG)) && \
    !defined(_UNIXSTL_NO_VERSION_WARNINGS)
#pragma message(__FILE__ "(81): This UNIXSTL file is pre-release 1.0. Please ensure you update as soon as UNIXSTL version 1.0 is available")
#endif /* !_DEBUG && !_COMSTL_NO_VERSION_WARNINGS */

#include <stlsoft.h>    // Currently depend on STLSoft libraries

#ifndef _INCL_UNIXSTL_H_UNIXSTL
/*#include <unixstl.h>*/    // Include the UNIXSTL root header
#endif /* !_INCL_UNIXSTL_H_UNIXSTL */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _UNIXSTL_NO_NAMESPACES
namespace unixstl
{
#endif /* _UNIXSTL_NO_NAMESPACES */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

// class performance_counter

class performance_counter
{
public:
    typedef performance_counter             class_type;

    typedef stlsoft_ns_qual(ss_sint64_t)    epoch_type;
    typedef stlsoft_ns_qual(ss_sint64_t)    interval_type;

// Construction
public:
    performance_counter();
    ~performance_counter() stlsoft_throw_0();

// Operations
public:
    void start();
    void stop();

// Attributes
public:
    interval_type   get_seconds() const;
    interval_type   get_milliseconds() const;
    interval_type   get_microseconds() const;
    
// Members
protected:
    timeval m_start;
    timeval m_end;
};

/* //////////////////////////////////////////////////////////////////////// */

inline performance_counter::performance_counter()
{
}

inline performance_counter::~performance_counter() stlsoft_throw_0()
{
}

inline void performance_counter::start()
{
    struct timezone tz;
    
    gettimeofday(&m_start, &tz);

    m_end   =   m_start;
}

inline void performance_counter::stop()
{
    struct timezone tz;
    
    gettimeofday(&m_end, &tz);
}

inline performance_counter::interval_type performance_counter::get_seconds() const
{
    interval_type   start   =   m_start.tv_sec + m_start.tv_usec / (1000 * 1000);
    interval_type   end     =   m_end.tv_sec   + m_end.tv_usec   / (1000 * 1000);

    return end - start;
}

inline performance_counter::interval_type performance_counter::get_milliseconds() const
{
    interval_type   start   =   m_start.tv_sec * 1000 + m_start.tv_usec / 1000;
    interval_type   end     =   m_end.tv_sec   * 1000 + m_end.tv_usec   / 1000;

    return end - start;
}

inline performance_counter::interval_type performance_counter::get_microseconds() const
{
    interval_type   start   =   m_start.tv_sec * 1000 * 1000 + m_start.tv_usec;
    interval_type   end     =   m_end.tv_sec   * 1000 * 1000 + m_end.tv_usec;

    return end - start;
}

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _UNIXSTL_NO_NAMESPACES
} // namespace unixstl
#endif /* _UNIXSTL_NO_NAMESPACES */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_INCL_UNIXSTL_H_UNIXSTL_PERFORMANCE_COUNTER */

/* ////////////////////////////////////////////////////////////////////////// */
